@@ -1,6 +1,6 @@
 from .bot_package_path import _set_bot_package_path, get_bot_package_path
 
-BOT_VERSION = (2, 5, 1)
+BOT_VERSION = (2, 6, 0)
 _set_bot_package_path(__path__[0])
 
 from .arena import *
@@ -22,7 +22,7 @@ from .api import *
 from .events import *
 from .disabled_commands import *
 from .duel import *
-from .command_server import start_command_server, SilentMessage, stop_command_server
+from .command_server import start_command_server, CommandServerMessage, stop_command_server
 from .modloader import *
 from .disabled_mods import *
 from .data import *
