@@ -1,16 +1,17 @@
-from sqlalchemy import create_engine, orm
-from sqlalchemy.ext.declarative import declarative_base
 import os
 
+from sqlalchemy import create_engine, orm
+from sqlalchemy.ext.declarative import declarative_base
 from ..config import database_cfg
+from ..util import is_env_key
 
 __all__ = ('Base', 'engine', 'get_database_session', 'DB_FILENAME', 'init_tables', 'session')
 
 
-def _try_get_env(value: str):
-    if value.lower().startswith('env_'):
+def _get_database_env_value(value: str):
+    if is_env_key(value):
         if value[4:] not in os.environ:
-            print('----------DATABASE ERROR----------')
+            print('----------ENVIRONMENT KEY ERROR----------')
             print(f'missing environment variable/key: {value[4:]}')
             input('\npress enter to exit...')
             exit(1)
@@ -24,13 +25,13 @@ try:
     engine = create_engine(f'sqlite:///{DB_FILENAME}'
                            if not database_cfg.enabled else
                            database_cfg.connection.format(
-                               database_format=_try_get_env(database_cfg.database_format),
-                               driver=_try_get_env(database_cfg.driver),
-                               username=_try_get_env(database_cfg.username),
-                               password=_try_get_env(database_cfg.password),
-                               address=_try_get_env(database_cfg.address),
-                               port=_try_get_env(database_cfg.port),
-                               database=_try_get_env(database_cfg.database)
+                               database_format=_get_database_env_value(database_cfg.database_format),
+                               driver=_get_database_env_value(database_cfg.driver),
+                               username=_get_database_env_value(database_cfg.username),
+                               password=_get_database_env_value(database_cfg.password),
+                               address=_get_database_env_value(database_cfg.address),
+                               port=_get_database_env_value(database_cfg.port),
+                               database=_get_database_env_value(database_cfg.database)
                            ),
                            pool_recycle=3600)
 except (ImportError, ModuleNotFoundError):
